require 'hawk'

module Transistor
  module Middleware
    class Authentication < Faraday::Middleware
      AUTHORIZATION_HEADER = 'Authorization'.freeze
      CONTENT_TYPE_HEADER = 'Content-Type'.freeze
      
      def initalize(app, options = {})
        super(app)
      end
      
      def call(env)
        env[:request_headers][AUTHORIZATION_HEADER] = authorize_request(env)
        @app.call(env)
      end
      
      private
      
      def authorize_request(env)
        Hawk::Client.build_authorization_header(
          credentials: Transistor.config.credentials,
          ts: Time.now.to_i,
          content_type: env[:request_headers][CONTENT_TYPE_HEADER].to_s.split(';').first,
          method: env[:method].to_s.upcase,
          port: env[:url].port || (env[:url].scheme == 'https' ? 443 : 80),
          host: env[:url].host,
          request_uri: env[:url].to_s.sub(%r{\A#{env[:url].scheme}://#{env[:url].host}(:#{env[:url].port})?}, '')
        )
      end
    end
  end
end